@@ -1,7 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
-    Copyright (C) 2007-2009 Piotr Eljasiak
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ struct GnomeCmdQuicksearchPopupPrivate
 
 inline void focus_file (GnomeCmdQuicksearchPopup *popup, GnomeCmdFile *f)
 {
-    if (strcmp (f->info->name, "..") == 0)
+    if (f->is_dotdot)
         return;
 
     popup->priv->last_focused_file = f;
@@ -138,11 +138,14 @@ static gboolean on_key_pressed (GtkWidget *entry, GdkEventKey *event, GnomeCmdQu
     switch (event->keyval)
     {
         case GDK_Escape:
-        case GDK_Return:
             popup->priv->fl->select_row(GNOME_CMD_CLIST (popup->priv->fl)->drag_motion_row);
             hide_popup (popup);
             return TRUE;
 
+        // for more convenience do ENTER action directly on the current quicksearch item
+        case GDK_Return:
+        case GDK_KP_Enter:
+
         // for more convenience jump direct to Fx function on the current quicksearch item
         case GDK_F3:
         case GDK_F4:
