@@ -1,6 +1,7 @@
 /*
     GNOME Commander - A GNOME based file manager
     Copyright (C) 2001-2006 Marcus Bjurman
+    Copyright (C) 2007-2010 Piotr Eljasiak
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +21,7 @@
 #include <config.h>
 #include <dirent.h>
 #include <gmodule.h>
+
 #include "gnome-cmd-includes.h"
 #include "gnome-cmd-data.h"
 #include "plugin_manager.h"
@@ -43,8 +45,7 @@ static GdkPixmap *blank_pixmap = NULL;
 static GdkBitmap *blank_mask = NULL;
 
 
-static void
-load_plugin (PluginData *data)
+static void load_plugin (PluginData *data)
 {
     GModule *module = g_module_open (data->fpath, G_MODULE_BIND_LAZY);
     PluginInfoFunc info_func;
@@ -109,8 +110,7 @@ load_plugin (PluginData *data)
 }
 
 
-static void
-activate_plugin (PluginData *data)
+static void activate_plugin (PluginData *data)
 {
     if (data->active)
         return;
@@ -130,8 +130,7 @@ activate_plugin (PluginData *data)
 }
 
 
-static void
-inactivate_plugin (PluginData *data)
+static void inactivate_plugin (PluginData *data)
 {
     if (!data->active)
         return;
@@ -142,11 +141,11 @@ inactivate_plugin (PluginData *data)
 }
 
 
-static void
-scan_plugins_in_dir (const gchar *dpath)
+static void scan_plugins_in_dir (const gchar *dpath)
 {
     DIR *dir = opendir (dpath);
-    char prev_dir[256];
+    char buff[256];
+    char *prev_dir;
     struct dirent *ent;
 
     if (dir == NULL)
@@ -157,7 +156,7 @@ scan_plugins_in_dir (const gchar *dpath)
         return;
     }
 
-    getcwd (prev_dir, sizeof(prev_dir));
+    prev_dir = getcwd (buff, sizeof(buff));
     chdir (dpath);
 
     while ((ent = readdir (dir)) != NULL)
@@ -203,7 +202,7 @@ scan_plugins_in_dir (const gchar *dpath)
 }
 
 
-void plugin_manager_init (void)
+void plugin_manager_init ()
 {
     if (plugins)
     {
@@ -243,7 +242,7 @@ void plugin_manager_init (void)
 }
 
 
-void plugin_manager_shutdown (void)
+void plugin_manager_shutdown ()
 {
     GList *out = NULL;
 
@@ -258,21 +257,19 @@ void plugin_manager_shutdown (void)
 }
 
 
-GList *plugin_manager_get_all (void)
+GList *plugin_manager_get_all ()
 {
     return plugins;
 }
 
 
-PluginData *
-get_selected_plugin (GtkCList *list)
+PluginData *get_selected_plugin (GtkCList *list)
 {
     return (PluginData *) gtk_clist_get_row_data (list, list->focus_row);
 }
 
 
-static void
-update_plugin_list (GtkCList *list, GtkWidget *dialog)
+static void update_plugin_list (GtkCList *list, GtkWidget *dialog)
 {
     gint old_focus = list->focus_row;
     gint row = 0;
@@ -299,7 +296,6 @@ update_plugin_list (GtkCList *list, GtkWidget *dialog)
         else
             gtk_clist_set_pixmap (list, row, 0, blank_pixmap, blank_mask);
 
-
         gtk_clist_set_row_data (list, row, data);
 
         row++;
@@ -309,8 +305,7 @@ update_plugin_list (GtkCList *list, GtkWidget *dialog)
 }
 
 
-static void
-do_toggle (GtkWidget *dialog)
+inline void do_toggle (GtkWidget *dialog)
 {
     GtkCList *list = GTK_CLIST (lookup_widget (dialog, "avail_list"));
     PluginData *data = get_selected_plugin (list);
@@ -364,15 +359,13 @@ on_plugin_unselected (GtkCList *list, gint row, gint column,
 }
 
 
-static void
-on_toggle (GtkButton *button, GtkWidget *dialog)
+static void on_toggle (GtkButton *button, GtkWidget *dialog)
 {
     do_toggle (dialog);
 }
 
 
-static void
-on_configure (GtkButton *button, GtkWidget *dialog)
+static void on_configure (GtkButton *button, GtkWidget *dialog)
 {
     GtkCList *list = GTK_CLIST (lookup_widget (dialog, "avail_list"));
     PluginData *data = get_selected_plugin (list);
@@ -384,8 +377,7 @@ on_configure (GtkButton *button, GtkWidget *dialog)
 }
 
 
-static void
-on_about (GtkButton *button, GtkWidget *dialog)
+static void on_about (GtkButton *button, GtkWidget *dialog)
 {
     GtkCList *list = GTK_CLIST (lookup_widget (dialog, "avail_list"));
     PluginData *data = get_selected_plugin (list);
@@ -399,14 +391,13 @@ on_about (GtkButton *button, GtkWidget *dialog)
 }
 
 
-static void
-on_close (GtkButton *button, GtkWidget *dialog)
+static void on_close (GtkButton *button, GtkWidget *dialog)
 {
     gtk_widget_destroy (dialog);
 }
 
 
-void plugin_manager_show (void)
+void plugin_manager_show ()
 {
     GtkWidget *dialog, *hbox, *bbox, *button;
     GtkWidget *avail_list;
@@ -460,7 +451,7 @@ void plugin_manager_show (void)
 
     update_plugin_list (GTK_CLIST (avail_list), dialog);
 
-    gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GNOME_STOCK_BUTTON_CLOSE, GTK_SIGNAL_FUNC(on_close), dialog);
+    gnome_cmd_dialog_add_button (GNOME_CMD_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_SIGNAL_FUNC(on_close), dialog);
     gnome_cmd_dialog_set_transient_for (GNOME_CMD_DIALOG (dialog), GTK_WINDOW (main_win));
 
     gtk_widget_set_size_request (GTK_WIDGET (dialog), 500, 300);
